[1720][forms] update for change in location of
[1720][forms] update for change in location of
Vocabulary get function

file:a/README.rst -> file:b/README.rst
--- a/README.rst
+++ b/README.rst
@@ -12,6 +12,8 @@
 * A custom Pylons controller for overriding some core CKAN behaviour
 
 * A custom Package edit form
+
+* A custom Group edit form
 
 * A plugin that allows for custom forms to be used for datasets based on 
   their "type".

--- a/ckanext/example/plugin.py
+++ b/ckanext/example/plugin.py
@@ -21,9 +21,11 @@
         found in the ``ini``-file.  Here we use it to specify the site
         title, and to tell CKAN to look in this package for templates
         and resources that customise the core look and feel.
+        
       - ``IGenshiStreamFilter`` allows us to filter and transform the
         HTML stream just before it is rendered.  In this case we use
         it to rename "frob" to "foobar"
+        
       - ``IRoutes`` allows us to add new URLs, or override existing
         URLs.  In this example we use it to override the default
         ``/register`` behaviour with a custom controller

--- a/ckanext/example/theme/templates/forms/dataset_form.html
+++ b/ckanext/example/theme/templates/forms/dataset_form.html
@@ -1,132 +1,192 @@
-<form id="package-edit" class="package_create_form ckan" method="post" 
+<form id="dataset-edit" method="post" 
   py:attrs="{'class':'has-errors'} if errors else {}"
   xmlns:i18n="http://genshi.edgewall.org/i18n"
   xmlns:py="http://genshi.edgewall.org/"
   xmlns:xi="http://www.w3.org/2001/XInclude">
 
+
 <div class="error-explanation" py:if="error_summary">
 <h2>Errors in form</h2>
 <p>The form contains invalid entries:</p>
 <ul>
-  <li py:for="key, error in error_summary.items()">${"%s: %s" % (key, error)}</li>
+  <li py:for="key, error in error_summary.items()">${"%s: %s" % (key, error)}
+    <py:if test="key=='Resources'">
+      <ul>
+        <py:for each="idx, errordict in enumerate(errors.get('resources', []))">
+          <li py:if="errordict">
+            Resource ${idx}:
+            <ul>
+              <li py:for="thiskey, thiserror in errordict.items()">${thiskey}: <py:for each="errorinfo in thiserror">${errorinfo}; </py:for></li>
+            </ul>
+          </li>
+        </py:for>
+      </ul>
+    </py:if>
+  </li>
 </ul>
 </div>
 
-<fieldset>
-  <legend>Basic information</legend>
-  <dl>
-    <dt><label class="field_req" for="title">Title *</label></dt>
-    <dd><input id="title" name="title" type="text" value="${data.get('title', '')}"/></dd>
-    <dd class="instructions basic">The title of the data set.</dd>
-    <dd class="instructions further">The main subject of the data should be clear. For cross-government data requirements, such as spend data, specify the public body the data belongs to or its geographical coverage, in order to distinguish your data from other similar datasets in data.gov.uk. If the data relates to a period of time, include that in the name, although this would not be appropriate for data which is updated over time. It is not a description - save that for the Abstract element. Do not give a trailing full stop.</dd>
-    <dd class="hints">e.g. Payments to suppliers with a value over &pound;500 from Harlow Council</dd>
-    <dd class="field_error" py:if="errors.get('title', '')">${errors.get('title', '')}</dd>
-
-    <dt><label class="field_req" for="name">Identifier *</label></dt>
-    <dd><input id="name" maxlength="100" name="name" type="text" value="${data.get('name', '')}" /></dd>
-    <dd class="instructions basic">A public unique identifier for the dataset</dd>
-    <dd class="instructions further">It should be roughly readable, with dashes separating words.</dd>
-    <dd class="hints">Format: Two or more lowercase alphanumeric, dash (-) or underscore (_) characters. e.g. uk-road-traffic-statistics-2008 or local-authority-spend-over-500-harlow</dd>
-    <dd class="field_error" py:if="errors.get('name', '')">${errors.get('name', '')}</dd>
-
-    <dt><label class="field_req" for="notes">Abstract *</label></dt>
-    <dd><textarea cols="60" id="notes" name="notes" rows="15">${data.get('notes', '')}</textarea></dd>
-    <dd class="instructions basic">The main description of the dataset</dd>
-    <dd class="instructions further">It is often displayed with the package title. In particular, it should start with a short sentence that describes the data set succinctly, because the first few words alone may be used in some views of the data sets. Here is the place to state if there are any limitations or deficiencies to the data in order to enable users to evaluate the information; even incomplete data may be adequate for some users.</dd>
-    <dd class="field_error" py:if="errors.get('notes', '')">${errors.get('notes', '')}</dd>
-  </dl>
-</fieldset>
-
-<fieldset>
-  <legend>Details</legend>
-  <dl>
-    <dt><label class="field_opt" for="date_released">Date released</label></dt>
-    <dd><input id="date_released" name="date_released" size="40" type="text" value="${data.get('date_released', '')}" /></dd>
-    <dd class="instructions basic">The date of the official release of the initial version of the dataset</dd>
-    <dd class="instructions further">This is the date that the data was officially released</dd>
-    <dd class="hints">DD/MM/YYYY</dd>
-    <dd class="field_error" py:if="errors.get('date_released', '')">${errors.get('date_released', '')}</dd>
-
-    <dt><label class="field_opt" for="date_updated">Date updated</label></dt>
-    <dd><input id="date_updated" name="date_updated" size="40" type="text" value="${data.get('date_updated', '')}"/></dd>
-    <dd class="instructions basic">The date of release of the most recent version of the dataset</dd>
-    <dd class="instructions further">This is not necessarily the date when it was updated on data.gov.uk. As with 'Date released', this is for updates to a particular dataset, such as corrections or refinements, not for that of a new time period.</dd>
-    <dd class="hints">DD/MM/YYYY</dd>
-    <dd class="field_error" py:if="errors.get('date_updated', '')">${errors.get('date_updated', '')}</dd>
-
-    <dt><label class="field_opt" for="date_update_future">Date to be published</label></dt>
-    <dd><input id="date_update_future" name="date_update_future" size="40" type="text" value="${data.get('date_update_future', '')}"/></dd>
-    <dd class="instructions basic">When the dataset will be updated in the future, if appropriate</dd>
-    <dd class="hints">DD/MM/YYYY</dd>
-    <dd class="field_error" py:if="errors.get('date_updated', '')">${errors.get('date_updated', '')}</dd>
-
-    <dt><label class="field_opt" for="update_frequency">Update frequency</label></dt>
-    <dd>
-      <select class="short" id="update_frequency" name="update_frequency">
-        <py:for each="freq_name, freq_desc in c.update_frequency">
-          <option value="${freq_name}" py:attrs="{'selected': 'selected' if data.get('update_frequency', '') == freq_name else None}" >
-            ${freq_desc}
-          </option>
+<fieldset id="basic-information">
+  <dl>
+    <dt class="title-label"><label class="field_opt" for="title">Title</label></dt>
+    <dd class="title-field">
+      <input id="title"
+        class="js-title"
+        name="title" type="text"
+        value="${data.get('title', '')}"
+        placeholder="${_('A short descriptive title for the dataset')}"
+      />
+    </dd>
+    <dd class="title-instructions field_error" py:if="errors.get('title', '')">${errors.get('title', '')}</dd>
+
+    <dt class="name-label"><label class="field_req" for="name">Url</label></dt>
+    <dd class="name-field">
+      <span class="js-url-text url-text">${url(controller='package', action='index')+'/'}<span class="js-url-viewmode js-url-suffix">&nbsp;</span><a href="#" style="display: none;" class="url-edit js-url-editlink js-url-viewmode">(edit)</a></span>
+      <input style="display: none;" id="name" maxlength="100" name="name" type="text" class="url-input js-url-editmode js-url-input" value="${data.get('name', '')}" />
+      <p class="js-url-is-valid">&nbsp;</p>
+    </dd>
+    <dd style="display: none;" class="js-url-editmode name-instructions basic">2+ characters, lowercase, using only 'a-z0-9' and '-_'</dd>
+    <dd class="js-url-editmode name-instructions field_error" py:if="errors.get('name', '')">${errors.get('name', '')}</dd>
+
+    <dt class="homepage-label"><label class="field_opt" for="url">Home Page</label></dt>
+    <dd class="homepage-field"><input id="url" name="url" type="text" value="${data.get('url', '')}"/></dd>
+    <dd class="homepage-instructions instructions basic">The URL for the web page describing the data (not the data itself).</dd>
+    <dd class="homepage-instructions hints">e.g. http://www.example.com/growth-figures.html</dd>
+    <dd class="homepage-instructions field_error" py:if="errors.get('url', '')">${errors.get('url', '')}</dd>
+
+    <dt class="license-label"><label class="field_opt" for="license_id">License</label></dt>
+    <dd class="license-field">
+      <select id="license_id" name="license_id">
+        <py:for each="licence_desc, licence_id in c.licences">
+          <option value="${licence_id}" py:attrs="{'selected': 'selected' if data.get('license_id', '') == licence_id else None}" >${licence_desc}</option>
         </py:for>
       </select>
-      <label class="inline" for="update_frequency-other">Other: 
-        <input class="medium-width" id="update_frequency-other" name="update_frequency-other" type="text" value="${data.get('update_frequency-other', '')}"/>
-      </label>
-    </dd>
-    <dd class="instructions basic">How frequently the dataset is updated with new versions</dd>
-    <dd class="instructions further">For one-off data, use 'never'. For those once updated but now discontinued, use 'discontinued'.</dd>
-    <dd class="field_error" py:if="errors.get('update_frequency', '')">${errors.get('update_frequency', '')}</dd>
-
-    <dt><label class="field_opt" for="precision">Precision</label></dt>
-    <dd><input id="precision" name="precision" size="40" type="text" value="${data.get('precision', '')}"/></dd>
-    <dd class="instructions basic">Indicate the level of precision in the data, to avoid over-interpretation.</dd>
-    <dd class="hints">e.g. 'per cent to two decimal places' or 'as supplied by respondents'</dd>
-
-    <dt><label class="field_opt" for="url">URL</label></dt>
-    <dd><input id="url" name="url" type="text" value="${data.get('url', '')}"/></dd>
-    <dd class="instructions basic">The Internet link to a web page discussing the dataset.</dd>
-    <dd class="hints">e.g. http://www.somedept.gov.uk/growth-figures.html</dd>
-    <dd class="field_error" py:if="errors.get('url', '')">${errors.get('url', '')}</dd>
-
-    <dt><label class="field_opt" for="taxonomy_url">Taxonomy URL</label></dt>
-    <dd><input id="taxonomy_url" name="taxonomy_url" size="40" type="text" value="${data.get('taxonomy_url', '')}"/></dd>
-    <dd class="instructions basic">An Internet link to a web page describing the taxonomies used in the dataset, if any, to ensure they understand any terms used.</dd>
-    <dd class="hints">e.g. http://www.somedept.gov.uk/growth-figures-technical-details.html</dd>
-    <dd class="field_error" py:if="errors.get('taxonomy_url', '')">${errors.get('taxonomy_url', '')}</dd>
-  </dl>
-  </fieldset>
-<fieldset>
-  <legend>Resources</legend>
-  <table class="flexitable">
+    </dd>
+    <dd class="license-instructions instructions basic">The licence under which the dataset is released.</dd>
+
+    <dt class="description-label"><label class="field_opt" for="notes">Description</label></dt>
+    <dd class="description-field"><div class="markdown-editor">
+      <ul class="button-row">
+        <li><button class="pretty-button js-markdown-edit depressed">Edit</button></li>
+        <li><button class="pretty-button js-markdown-preview">Preview</button></li>
+      </ul>
+      <textarea class="markdown-input" name="notes" id="notes" placeholder="${_('Start with a summary sentence ...')}">${data.get('notes','')}</textarea>
+      <div class="markdown-preview" style="display: none;"></div>
+      <span class="hints">You can use <a href="http://daringfireball.net/projects/markdown/syntax" target="_blank">Markdown formatting</a> here.</span>
+      <!--
+      <dd class="instructions basic">The main description of the dataset</dd>
+      <dd class="instructions further">It is often displayed with the dataset title. In particular, it should start with a short sentence that describes the dataset succinctly, because the first few words alone may be used in some views of the datasets.</dd>
+      -->
+    </div></dd>
+  </dl>
+</fieldset>
+
+<fieldset id="resources">
+  <div class="instructions basic"><h3>Resources: the files and APIs associated with this dataset</h3></div>
+  <table class="resource-table-edit">
     <thead>
       <tr>
-        <th class="field_req resource-url">URL*</th>
-        <th class="field_opt resource-format">Format</th>
-        <th class="field_opt resource-description">Description</th>
-        <th class="field_opt resource-hash">Hash</th>
+        <th class="field_req resource-url">Resource</th>
+        <th class="resource-delete-link"></th>
       </tr>
     </thead>
-    <tbody>
-      <py:for each="num, res in enumerate(data.get('resources', []) + [{}])">
-      <tr>
-        <py:for each="col in c.resource_columns">
-        <td class="resource-${col}">
-          <input name="resources__${num}__${col}" type="text" value="${res.get(col, '')}" class="${'medium-width' if col=='description' else 'short'}" />
-        </td>
-        </py:for>
-        <td class="resource-id"><input name="resources__${num}__id" type="hidden" value="${res.get('id', '')}" /></td>
-      </tr>
-      </py:for>
+    <tbody class="js-resource-editor">
     </tbody>
   </table>
-  <div class="instructions basic">The files containing the data or address of the APIs for accessing it</div>
-  <div class="instructions further">These can be repeated as required. For example if the data is being supplied in multiple formats, or split into different areas or time periods, each file is a different 'resource' which should be described differently. They will all appear on the dataset page on data.gov.uk together.<br/> <b>URL:</b> This is the Internet link directly to the data - by selecting this link in a web browser, the user will immediately download the full data set. Note that datasets are not hosted by data.gov.uk, but by the responsible department<br/> e.g. http://www.somedept.gov.uk/growth-figures-2009.csv<br/><b>Format:</b> This should give the file format in which the data is supplied. You may supply the data in a form not listed here, constrained by the <a href="http://data.gov.uk/blog/new-public-sector-transparency-board-and-public-data-transparency-principles" target="_blank">Public Sector Transparency Board's principles</a> that require that all data is available in an 'open and standardised format' that can be read by a machine. Data can also be released in formats that are not machine-processable (e.g. PDF) alongside this.<br/></div>
-  <div class="hints">Format choices: CSV | RDF | XML | XBRL | SDMX | HTML+RDFa | Other as appropriate</div>
-  <div class="field_error" py:if="errors.get('resources', '')">Package resource(s) incomplete.</div>
-</fieldset>
-
-<fieldset>
-  <legend>More details</legend>
+
+
+  <div class="resource-add">
+    <ul class="button-row">
+      <li><h4>Add a resource:</h4></li>
+      <li><button class="pretty-button js-link-file">Link to a file</button></li>
+      <li><button class="pretty-button js-link-api">Link to an API</button></li>
+      <li class="js-upload-file ckan-logged-in" style="display: none;"><button class="pretty-button js-upload-file">Upload a file</button></li>
+    </ul>
+  </div>
+</fieldset>
+
+<fieldset id="groups">
+  <h3>Groups</h3>
+  <dl>
+    <py:for each="num, group in enumerate(data.get('groups', []))">
+        <?python
+        authorized_group = [group_authz for group_authz in c.groups_authz if group_authz['id'] == group['id']]
+        authorized_group = authorized_group[0] if authorized_group else None
+        ?>
+
+      <dt py:if="'id' in group">
+      <input type="${'checkbox' if authorized_group else 'hidden'}" name="groups__${num}__id" checked="checked" value="${group['id']}" />
+      <input type="hidden" name="groups__${num}__name" value="${group.get('name', authorized_group['name'] if authorized_group else '')}" />
+      </dt>     
+      <dd py:if="'id' in group"><label for="groups__${num}__checked">${group.get('name', authorized_group['name'] if authorized_group else '')}</label></dd>
+    </py:for>
+
+    <dt>Group</dt>
+    <dd py:if="c.groups_available"> 
+      <select id="groups__${len(data.get('groups', []))}__id" name="groups__${len(data.get('groups', []))}__id">
+        <option selected="selected" value="">(None)</option>
+        <py:for each="group in c.groups_available">
+        <option value="${group['id']}" >${group['name']}</option>
+        </py:for>
+      </select>
+    </dd> 
+    <dd py:if="not c.groups_available"><em>Cannot add any groups.</em></dd>
+  </dl>
+  <h3>Tags</h3>
+  <dl>
+    <dt class="tags-label"><label class="field_opt" for="tags">Tags</label></dt>
+    <dd class="tags-field">
+      <input class="long autocomplete-tag" id="tag_string" name="tag_string" size="60" type="text" 
+               value="${data.get('tag_string') or ', '.join([tag['name'] for tag in data.get('tags', [])])}" />
+    </dd>
+    <dd class="tags-instructions instructions basic" i18n:msg="">Comma-separated terms that may link this dataset to similar ones. For more information on conventions, see <a href="http://wiki.okfn.org/ckan/doc/faq#TagConventions">this wiki page</a>.</dd>
+    <dd class="tags-instructions hints">e.g. pollution, rivers, water quality</dd>
+    <dd class="tags-instructions field_error" py:if="errors.get('tag_string', '')">${errors.get('tag_string', '')}</dd>
+  </dl>
+  <h3>Tags with Vocabulary</h3>
+  <dl>
+    <dt class="tags-label"><label class="field_opt" for="tags_vocab">Tags</label></dt>
+    <dd class="tags-field">
+      <input class="long autocomplete-tag" id="vocab_tag_string" name="vocab_tag_string" size="60" type="text" 
+               value="${data.get('vocab_tag_string') or ', '.join([tag['name'] for tag in data.get('vocab_tags', [])])}" />
+    </dd>
+    <dd class="tags-instructions instructions basic" i18n:msg="">Similar to standard CKAN tags, but these use a fixed vocabulary.</dd>
+    <dd class="tags-instructions field_error" py:if="errors.get('vocab_tag_string', '')">${errors.get('vocab_tag_string', '')}</dd>
+  </dl>
+</fieldset>
+<fieldset id='further-information'>
+  <dl>
+    <dt><label class="field_opt" for="author">Author</label></dt>
+    <dd><input id="author" name="author" type="text" value="${data.get('author', '')}" /></dd>
+    <dd class="instructions basic">The name of the main contact, for enquiries about this particular dataset, using the e-mail address in the following field.</dd>
+
+    <dt><label class="field_opt" for="author_email">Author email</label></dt>
+    <dd><input id="author_email" name="author_email" type="text" value="${data.get('author_email', '')}" /></dd>
+
+    <dt><label class="field_opt" for="maintainer">Maintainer</label></dt>
+    <dd><input id="maintainer" name="maintainer" type="text" value="${data.get('maintainer', '')}" /></dd>
+    <dd class="instructions basic">If there is another important contact person (in addition to the person in the Author field) then provide details here.</dd>
+
+    <dt><label class="field_opt" for="maintainer_email">Maintainer email</label></dt>
+    <dd><input id="maintainer_email" name="maintainer_email" type="text" value="${data.get('maintainer_email', '')}" /></dd>
+
+    <dt><label class="field_opt" for="version">Version</label></dt>
+    <dd><input id="version" maxlength="100" name="version" type="text" value="${data.get('version', '')}" /></dd>
+    <dd class="instructions basic">A number representing the version (if applicable)</dd>
+    <dd class="hints">e.g. 1.2.0</dd>
+
+    <dt py:if="c.is_sysadmin or c.auth_for_change_state"><label class="field_opt" for="state">State</label></dt>
+    <dd py:if="c.is_sysadmin or c.auth_for_change_state">
+    <select id="state" name="state" >
+      <option py:attrs="{'selected': 'selected' if data.get('state') == 'active' else None}" value="active">active</option>
+      <option py:attrs="{'selected': 'selected' if data.get('state') == 'deleted' else None}" value="deleted">deleted</option>
+    </select>
+    </dd>
+
+  </dl>
+</fieldset>
+
+<fieldset id='extras'>
   <dl>
     <dt><label class="field_req" for="published_by">Published by *</label></dt>
     <dd>
@@ -138,93 +198,54 @@
         </py:for>
       </select>
     </dd>
-    <dd class="instructions basic">The organisation (usually a public body) credited with or associated with the publication of this data.</dd>
-    <dd class="instructions further">Often datasets are associated with both a government department and an outside agency, in which case this field should store the department and "Published via" should store the agency. When an organisation is not listed, please request it using the form found in your data.gov.uk user page under the "Publishers" tab. An asterisk (*) denotes an pre-existing value for this field, which is allowed, but the current user's permissions would not be able to change a package\s publisher to this value.</dd>
-
-    <dt><label class="field_opt" for="published_via">Published via</label></dt>
+    <dd class="instructions basic">The organisation credited with or associated with the publication of this data.</dd>
+
+    <py:with vars="extras = data.get('extras', [])">
+    <py:for each="num, extra in enumerate(data.get('extras', []))">
+    <dt><label for="extras__${num}__value">${extra.get('key')}</label></dt>
     <dd>
-      <select id="published_via" name="published_via">
-        <py:for each="published_name, published_desc in c.publishers">
-          <option value="${published_name}" py:attrs="{'selected': 'selected' if data.get('published_via', '') == published_name else None}" >
-            ${published_desc}
-          </option>
-        </py:for>
-      </select>
-    </dd>
-    <dd class="instructions basic">A second organisation that is credited with or associated with the publication of this data.</dd>
-    <dd class="instructions further">Often datasets are associated with both a government department and an outside agency, in which case the "Published by" field should store the department and this field should store the agency. When an organisation is not listed, please request it using the form found in your data.gov.uk user page under the "Publishers" tab. An asterisk (*) denotes an pre-existing value for this f
--- a/ckanext/example/forms.py
+++ b/ckanext/example/forms.py
@@ -1,43 +1,28 @@
 import os, logging
 from ckan.authz import Authorizer
-import ckan.logic.action.create as create
-import ckan.logic.action.update as update
-import ckan.logic.action.get as get
-from ckan.logic.converters import date_to_db, date_to_form, convert_to_extras, convert_from_extras
-from ckan.logic import NotFound, NotAuthorized, ValidationError
-from ckan.logic import tuplize_dict, clean_dict, parse_params
-import ckan.logic.schema as default_schema
-from ckan.logic.schema import package_form_schema
-import ckan.logic.validators as val
-from ckan.lib.base import BaseController, render, c, model, abort, request
-from ckan.lib.base import redirect, _, config, h
-from ckan.lib.package_saver import PackageSaver
-from ckan.lib.field_types import DateType, DateConvertError
-from ckan.lib.navl.dictization_functions import Invalid
-from ckan.lib.navl.dictization_functions import validate, missing
-from ckan.lib.navl.dictization_functions import DataError, flatten_dict, unflatten
-from ckan.plugins import IDatasetForm, IGroupForm, IConfigurer
+from ckan.logic.converters import convert_to_extras,\
+    convert_from_extras, convert_to_tags, convert_from_tags
+from ckan.logic.schema import package_form_schema, group_form_schema
+from ckan.model import vocabulary
+from ckan.lib.base import c, model
+from ckan.plugins import IDatasetForm, IGroupForm, IConfigurer, IConfigurable
 from ckan.plugins import implements, SingletonPlugin
-
-from ckan.lib.navl.validators import (ignore_missing,
-                                      not_empty,
-                                      empty,
-                                      ignore,
-                                      keep_extras,
-                                     )
+from ckan.lib.navl.validators import ignore_missing, not_empty, keep_extras
 
 log = logging.getLogger(__name__)
 
 class ExampleGroupForm(SingletonPlugin):
-    """This plugin demonstrates how a theme packaged as a CKAN
-    extension might extend CKAN behaviour.
-
-    In this case, we implement twos extension interfaces to provide custom 
+    """This plugin demonstrates how a class packaged as a CKAN
+    extension might extend CKAN behaviour by providing custom forms
+    based on the type of a Group.
+
+    In this case, we implement two extension interfaces to provide custom 
     forms for specific types of group.
 
       - ``IConfigurer`` allows us to override configuration normally
         found in the ``ini``-file.  Here we use it to specify where the
         form templates can be found.
+        
       - ``IGroupForm`` allows us to provide a custom form for a dataset
         based on the 'type' that may be set for a group.  Where the 
         'type' matches one of the values in group_types then this 
@@ -63,8 +48,6 @@
         Returns a string representing the location of the template to be
         rendered.  e.g. "forms/group_form.html".
         """        
-        from pdb import set_trace; set_trace()
-        
         return 'forms/group_form.html'
 
     def group_types(self):
@@ -78,7 +61,7 @@
         attempts to register more than one plugin instance to a given group
         type will raise an exception at startup.
         """
-        return ["example"]
+        return ["testgroup"]
 
     def is_fallback(self):
         """
@@ -90,7 +73,31 @@
         """
         return False                
                 
-                
+    def form_to_db_schema(self):
+        """
+        Returns the schema for mapping group data from a form to a format
+        suitable for the database.
+        """
+        return group_form_schema()
+
+    def db_to_form_schema(self):
+        """
+        Returns the schema for mapping group data from the database into a
+        format suitable for the form (optional)
+        """
+        return {}
+        
+    def check_data_dict(self, data_dict):
+        """
+        Check if the return data is correct.
+
+        raise a DataError if not.
+        """
+
+    def setup_template_variables(self, context, data_dict):
+        """
+        Add variables to c just prior to the template being rendered.
+        """                
                 
 
 class ExampleDatasetForm(SingletonPlugin):
@@ -109,6 +116,7 @@
     """
     implements(IDatasetForm, inherit=True)
     implements(IConfigurer, inherit=True)    
+    implements(IConfigurable)
     
     def update_config(self, config):
         """
@@ -122,6 +130,19 @@
         config['extra_template_paths'] = ','.join([template_dir,
                 config.get('extra_template_paths', '')])
 
+    def configure(self, config):
+        """
+        Adds our new vocabulary to the database if it doesn't
+        already exist.
+        """
+        self.vocab_name = u'example_vocab'
+        v = model.Vocabulary.get(self.vocab_name)
+        if not v:
+            log.info("Adding vocab %s" % self.vocab_name)
+            vocab = model.Vocabulary(self.vocab_name)
+            model.Session.add(vocab)
+            model.Session.commit()
+
     def package_form(self):
         """
         Returns a string representing the location of the template to be
@@ -137,7 +158,7 @@
         As this is not the fallback controller we should return False.  If 
         we were wanting to act as the fallback, we'd return True
         """
-        return False
+        return True
 
     def package_types(self):
         """
@@ -150,7 +171,7 @@
         attempts to register more than one plugin instance to a given package
         type will raise an exception at startup.
         """
-        return ["example"]
+        return ["example_dataset_form"]
 
     def setup_template_variables(self, context, data_dict=None):
         """
@@ -173,65 +194,26 @@
         Returns the schema for mapping package data from a form to a format
         suitable for the database.
         """
-        schema = {
-            'title': [not_empty, unicode],
-            'name': [not_empty, unicode, val.name_validator, val.package_name_validator],
-            'notes': [not_empty, unicode],
-
-            'date_released': [date_to_db, convert_to_extras],
-            'date_updated': [date_to_db, convert_to_extras],
-            'date_update_future': [date_to_db, convert_to_extras],
-            'url': [unicode],
-            'taxonomy_url': [unicode, convert_to_extras],
-
-            'resources': default_schema.default_resource_schema(),
-            
+        schema = package_form_schema()
+        schema.update({
             'published_by': [not_empty, unicode, convert_to_extras],
-            'published_via': [ignore_missing, unicode, convert_to_extras],
-            'author': [ignore_missing, unicode],
-            'author_email': [ignore_missing, unicode],
-            'mandate': [ignore_missing, unicode, convert_to_extras],
-            'license_id': [ignore_missing, unicode],
-            'tag_string': [ignore_missing, val.tag_string_convert],
-            'national_statistic': [ignore_missing, convert_to_extras],
-            'state': [val.ignore_not_admin, ignore_missing],
-
-            'log_message': [unicode, val.no_http],
-
-            '__extras': [ignore],
-            '__junk': [empty],
-        }
+            'vocab_tag_string': [ignore_missing, convert_to_tags(self.vocab_name)],
+        })
         return schema
     
-    def db_to_form_schema(data):
+    def db_to_form_schema(self):
         """
         Returns the schema for mapping package data from the database into a
         format suitable for the form (optional)
         """
-        schema = {
-            'date_released': [convert_from_extras, ignore_missing, date_to_form],
-            'date_updated': [convert_from_extras, ignore_missing, date_to_form],
-            'date_update_future': [convert_from_extras, ignore_missing, date_to_form],
-            'precision': [convert_from_extras, ignore_missing],
-            'taxonomy_url': [convert_from_extras, ignore_missing],
-
-            'resources': default_schema.default_resource_schema(),
-            'extras': {
-                'key': [],
-                'value': [],
-                '__extras': [keep_extras]
-            },
+        schema = package_form_schema()
+        schema.update({
             'tags': {
                 '__extras': [keep_extras]
             },
-            
+            'vocab_tag_string': [convert_from_tags(self.vocab_name), ignore_missing],
             'published_by': [convert_from_extras, ignore_missing],
-            'published_via': [convert_from_extras, ignore_missing],
-            'mandate': [convert_from_extras, ignore_missing],
-            'national_statistic': [convert_from_extras, ignore_missing],
-            '__extras': [keep_extras],
-            '__junk': [ignore],
-        }
+        })
         return schema
 
     def check_data_dict(self, data_dict):
